@@ -9,7 +9,9 @@ from .utils import rfunctions as functions
 
 
 class Rule(ABC):
-    def __init__(self, condition: Optional[Condition] = None, activation: Optional[Activation] = None):
+    def __init__(
+        self, condition: Optional[Condition] = None, activation: Optional[Activation] = None,
+    ):
 
         if condition is not None and not isinstance(condition, Condition):
             raise TypeError("Argument 'condition' must derive from Condition or be None.")
@@ -17,19 +19,12 @@ class Rule(ABC):
             raise TypeError("Argument 'activation' must derive from Activation or be None.")
 
         self._condition = condition
-
         self._activation = activation
-        self._coverage = None
         self._prediction = None
-        self._std = None
-        self._criterion = None
 
         self._time_fit = -1
-        self._time_calc_criterion = -1
-        self._time_calc_prediction = -1
-        self._time_predict = -1
-        self._time_calc_std = -1
         self._time_calc_activation = -1
+        self._time_predict = -1
 
     def __and__(self, other: "Rule") -> "Rule":
         condition = self._condition + other._condition
@@ -66,14 +61,6 @@ class Rule(ABC):
     def prediction(self) -> float:
         return self._prediction
 
-    @property
-    def std(self) -> float:
-        return self._std
-
-    @property
-    def criterion(self) -> float:
-        return self._criterion
-
     @property
     def time_fit(self):
         return self._time_fit
@@ -82,18 +69,6 @@ class Rule(ABC):
     def time_predict(self):
         return self._time_predict
 
-    @property
-    def time_calc_prediction(self):
-        return self._time_calc_prediction
-
-    @property
-    def time_calc_criterion(self):
-        return self._time_calc_criterion
-
-    @property
-    def time_calc_std(self):
-        return self._time_calc_std
-
     @property
     def time_calc_activation(self):
         return self._time_calc_activation
@@ -130,11 +105,72 @@ class Rule(ABC):
         """Computes activation, prediction, std and criteria of the rule for a given xs and y."""
         t0 = time()
         self.calc_activation(xs)  # returns Activation
+        self.calc_attributes(xs, y, **kwargs)
+        self._time_fit = time() - t0
+
+    def calc_attributes(self, xs: np.ndarray, y: np.ndarray, **kwargs):
+        """Implement in daughter class"""
+        pass
+
+    def calc_activation(self, xs: np.ndarray) -> None:
+        t0 = time()
+        self._activation = self.evaluate(xs)
+        self._time_calc_activation = time() - t0
+
+    def predict(self, xs: Optional[np.ndarray] = None) -> np.ndarray:
+        """Returns the prediction vector. If xs is not given, will use existing activation vector.
+        Will raise ValueError is xs is None and activation is not yet known."""
+        t0 = time()
+        if xs is not None:
+            self.calc_activation(xs)
+        elif self.activation is None:
+            raise ValueError("If the activation vector has not been computed yet, xs can not be None.")
+        to_ret = self._prediction * self.activation
+        self._time_predict = time() - t0
+        return to_ret
+
+
+class RegressionRule(Rule):
+    def __init__(
+        self, condition: Optional[Condition] = None, activation: Optional[Activation] = None,
+    ):
+        super().__init__(condition, activation)
+
+        self._coverage = None
+        self._std = None
+        self._criterion = None
+
+        # Inspection / Audit attributs
+        self._time_calc_criterion = -1
+        self._time_calc_prediction = -1
+        self._time_calc_std = -1
+
+    @property
+    def std(self) -> float:
+        return self._std
+
+    @property
+    def criterion(self) -> float:
+        return self._criterion
+
+    @property
+    def time_calc_prediction(self):
+        return self._time_calc_prediction
+
+    @property
+    def time_calc_criterion(self):
+        return self._time_calc_criterion
+
+    @property
+    def time_calc_std(self):
+        return self._time_calc_std
+
+    def calc_attributes(self, xs: np.ndarray, y: np.ndarray, crit: str = "mse", **kwargs):
+        self.calc_activation(xs)
         self.calc_prediction(y)
         self.calc_std(y)
         prediction_vector = self.prediction * self.activation
         self.calc_criterion(prediction_vector, y, crit)
-        self._time_fit = time() - t0
 
     def calc_activation(self, xs: np.ndarray) -> None:
         t0 = time()
@@ -159,7 +195,7 @@ class Rule(ABC):
 
     def calc_criterion(self, p, y, c, **kwargs):
         t0 = time()
-        self._criterion = functions.calc_criterion(p, y, c)
+        self._criterion = functions.calc_regression_criterion(p, y, c)
         self._time_calc_criterion = time() - t0
 
     def predict(self, xs: Optional[np.ndarray] = None) -> np.ndarray:
@@ -173,3 +209,53 @@ class Rule(ABC):
         to_ret = self._prediction * self.activation
         self._time_predict = time() - t0
         return to_ret
+
+
+class ClassificationRule(Rule):
+    def __init__(
+        self, condition: Optional[Condition] = None, activation: Optional[Activation] = None,
+    ):
+        super().__init__(condition, activation)
+
+        self._criterion = None
+
+        # Inspection / Audit attributs
+        self._time_calc_criterion = -1
+        self._time_calc_prediction = -1
+
+    @property
+    def prediction(self) -> Union[int, str]:
+        return self._prediction
+
+    @property
+    def criterion(self) -> float:
+        return self._criterion
+
+    def calc_attributes(self, xs: np.ndarray, y: np.ndarray, crit: str = "success_rate", **kwargs):
+        self.calc_prediction(y)
+        self.calc_criterion(y, crit)
+
+    def calc_prediction(self, y: np.ndarray) -> None:
+        """If you do not need to to all 'fit' but only want to compute 'prediction'"""
+        t0 = time()
+        if self.activation is None:
+            raise ValueError("The activation vector has not been computed yet.")
+        self._prediction = functions.most_common_class(self.activation, y)
+        self._time_calc_prediction = time() - t0
+
+    def calc_criterion(self, y, c):
+        t0 = time()
+        self._criterion = functions.calc_classification_criterion(self.activation, self.prediction, y, c)
+        self._time_calc_criterion = time() - t0
+
+    def predict(self, xs: Optional[np.ndarray] = None) -> np.ndarray:
+        """Returns the prediction vector. If xs is not given, will use existing activation vector.
+        Will raise ValueError is xs is None and activation is not yet known."""
+        t0 = time()
+        if xs is not None:
+            self.calc_activation(xs)
+        elif self.activation is None:
+            raise ValueError("If the activation vector has not been computed yet, xs can not be None.")
+        to_ret = np.array([self._prediction if i == 1 else "" for i in self.activation])
+        self._time_predict = time() - t0
+        return to_ret
